// Fig. 15.13: RandomLetters.cs
// Writes a random letter to a label
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Threading;

public class RandomLetters
{
   private static Random generator = new Random(); // for random letters
   private bool suspended = false; // true if thread is suspended
   private Label output; // Label to display output
   private string threadName; // name of the current thread

   // RandomLetters constructor
   public RandomLetters( Label label )
   {
      output = label;
   } // end 

   // delegate that allows method DisplayCharacter to be called
   // in the thread that creates and maintains the GUI       
   private delegate void DisplayDelegate( char displayChar );  

   // method DisplayCharacter sets the Label's Text property
   private void DisplayCharacter( char displayChar )
   {
      // output character in Label
      output.Text = threadName + ": " + displayChar;
   } // end method DisplayCharacter
   
   // place random characters in GUI
   public void GenerateRandomCharacters()
   {
      // get name of executing thread
      threadName = Thread.CurrentThread.Name;

      while ( true ) // infinite loop; will be terminated from outside
      {
         // sleep for up to 1 second
         Thread.Sleep( generator.Next( 1001 ) );

         lock ( this ) // obtain lock
         {
            while ( suspended ) // loop until not suspended
            {
               Monitor.Wait( this ); // suspend thread execution
            } // end while
         } // end lock

         // select random uppercase letter
         char displayChar = ( char ) ( generator.Next( 26 ) + 65 );

         // display character on corresponding Label
         output.Invoke( new DisplayDelegate( DisplayCharacter ),
            new object[] { displayChar } );
      } // end while
   } // end method GenerateRandomCharacters

   // change the suspended/running state
   public void Toggle()
   {
      suspended = !suspended; // toggle bool controlling state

      // change label color on suspend/resume
      output.BackColor = suspended ? Color.Red : Color.LightGreen;

      lock ( this ) // obtain lock
      {
         if ( !suspended ) // if thread resumed
            Monitor.Pulse( this );
      } // end lock
   } // end method Toggle
} // end class RandomLetters

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/